# Copyright (c) 2012 <Jaume Devesa (jaumedevesa@gmail.com)>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# 'actions' module is the responsible to execute the business logic of each one
# of the available actions, call the GithubFacade to retrieve the
# Github API response, and handles errors and responses.

from utils import download, build_result, GistsConfigurer
import literals
import model
import os

"""

gists.actions
~~~~~~~~~~~~~~~~

Actions is the main module of the package. It receives data from the 'handlers'
module, call to GitHub Gists API and manage the response.


"""


def list_gists(username, facade, want_starred):
    """ Retrieve the list of gists for a concrete user.

    :param token: GitHub authentication token
    :param facade: instance of the object that actually performs the request
    :param want_starred: if we want the starred ones
    """

    if not want_starred:
        response = facade.request_list_of_gists(username)
    else:
        response = facade.request_list_starred_gists(username)

    if response.ok:
        # List of gists for the requested user found.
        list_gists = []
        for gist in response.json:
            list_gists.append(model.Gist(gist))

        return build_result(True, list_gists)
    else:
        # GitHub response error. Parse the response
        return build_result(False, literals.LISTS_ERROR,
                            response.json['message'])


def get(gist_id, requested_file, destination_dir, facade):
    """ Download a gist file.

    Gists can have several files. This method searches for and downloads
    a single file from a gist.
    If the 'requested_file' is not informed, then it won't raise an error
    only if the gist have just a single file.

    :param gist_id: identifier of the gist to download
    :param requested_file: name of the Gist file to download
    :param destination_dir: destination directory after the download
    :param facade: instance of the object that actually perform the request
    """

    # Get the gist information
    response = facade.request_gist(gist_id)

    if response.ok:
        # Gist file found. Parse it into a 'model.Gist' class.
        gist_obj = model.Gist(response.json)
        list_names = [gistfile.filename for gistfile in gist_obj.files]

        if len(gist_obj.files) == 1 and not requested_file:
            # Download the only file in the gist
            gistfile = gist_obj.files[0]
            download(gistfile.raw_url, destination_dir,
                     gistfile.filename, gistfile.size)

            result = build_result(True, literals.DOWNLOAD_OK,
                                  gistfile.filename)
        else:
            # Gist have more than one file and filename not specified. Error
            if not requested_file:
                list_names = ", ".join(list_names)
                result = build_result(False, literals.DOWNLOAD_MORE_FILES,
                                      list_names)
            else:
                # Search for the Gist file
                gistfile = gist_obj.getFile(requested_file)
                if gistfile:

                    # Gist file found. Download it.
                    download(gistfile.raw_url, destination_dir,
                             gistfile.filename, gistfile.size)

                    result = build_result(True, literals.DOWNLOAD_OK,
                                          gistfile.filename)
                else:
                    # Requested file not found in Gist
                    list_of_names = ", ".join(list_names)
                    result = build_result(False, literals.FILE_NOT_FOUND,
                                          list_of_names)

    else:
        # Handle GitHub response error
        result = build_result(False, literals.DOWNLOAD_ERROR,
                              response.json['message'])

    return result


def show(gist_id, requested_file, facade):
    """ Retrieve a single gist.

    If the 'requested_file' is None, then it will show the
    'metadata' of the Gist. This is: its description, urls, file names..

    If the 'requested_file' is informed, then it will show
    the content of the gist file.

    :param gist_id: identifier of the Gist to print
    :param requested_file: Gist File to show
    :param facade: instance of the object that actually performs the request
    """

    # get the gist information
    response = facade.request_gist(gist_id)

    if response.ok:
        # Gist found. Parse the json response into the 'model.Gist' class
        gist_obj = model.Gist(response.json)
        if not requested_file:
            # Fill the response with the metadata of the gist
            result = build_result(True, gist_obj)
        else:
            # We want to return the content of the file. Search for the content
            list_names = [gistfile.filename for gistfile in gist_obj.files]
            file_gist = gist_obj.getFile(requested_file)
            if file_gist:
                # Fill the response with the metadata of the gist
                result = build_result(True, file_gist)
            else:
                # File not found in Gist
                list_of_names = ", ".join(list_names)
                result = build_result(False, literals.FILE_NOT_FOUND,
                                      list_of_names)

    else:
        # GitHub response not ok. Parse the response
        result = build_result(False, literals.SHOW_ERROR,
                              response.json['message'])

    return result


def post(public, upload_files, filepath, description, facade):
    """ Create a new Gist.

    Currently only support create Gist with single files. (Then you can
    'update' the gist and attach more files in it, but the creation only
    supports one file)

    You are able to specify if you want to create a public or private
    gist and set its description.

    :param public: whenever new Gist should be public or private
    :param upload_files: list with  input files to upload
    :param filepath: input parameter path
    :param description: brief description of the Gist
    :param facade: instance of the object that actually performs the request
    """

    # Prepare the Gist file object and set its description and 'public' value
    gist = model.Gist()
    if description:
        gist.description = description
    gist.public = public

    for upfile in upload_files:
        # Prepare the content reading the file
        gistFile = model.GistFile()
        gistFile.filename = upfile
        with open(os.path.join(filepath, upfile), 'r') as f:
            file_content = f.read()
            gistFile.content = file_content
        gist.addFile(gistFile)

    response = facade.create_gist(gist)
    # Parse the response
    if response.ok:
        result = build_result(True, model.Gist(response.json))
    else:
        if response.json:
            result = build_result(False, response.json['message'])
        else:
            result = build_result(False, literals.UNHANDLED_EXCEPTION)
    return result


def delete(gistid, facade):
    """ Just deletes a gist.

    :param gistid: identifier of the Gist to delete
    :param facade: instance of the object that actually performs the request
    """

    # First check if the gist exists
    response = facade.request_gist(gistid)

    if response.ok:

        # Gist Found. Ask for confirmation
        value_raw_input = (literals.DELETE_CONFIRMATION) % (gistid)
        value = raw_input(value_raw_input)
        accepted_values_for_yes = ["y", "yes", "ofcourse", "ye"]
        if value.lower() in accepted_values_for_yes:

            # Perform the deletion
            response = facade.delete_gist(gistid)
            if response.ok:
                result = build_result(True, literals.DELETE_OK, gistid)

            else:
                res_message = response.json['message']
                result = build_result(False, literals.DELETE_NOK, res_message)
        else:
            # Aborted mission
            result = build_result(False, literals.DELETE_ABORTED)
    else:
        # Gist not retrieved.
        res_message = response.json['message']
        result = build_result(False, literals.DELETE_NOK, res_message)

    return result


def update(gistid, description, filenames, filepath, new, remove, facade):
    """ Updates a gist.

    :param gistid: identifier of the Gist to update
    :param description: new description of the Gist. If 'None' it won't be
    updated
    :param filenames: list with names of the files to modify its contents
    :param filepath: input parameter path
    :param new: whenever the file is new or already exists
    :param remove: if the file should be deleted instead of modified
    :param facade: instance of the object that actually performs the request
    """

    # First get the result
    response = facade.request_gist(gistid)

    if response.ok:
        # Gist found.
        gist = model.Gist(response.json)
    else:
        result = build_result(False, literals.UPDATE_NOK,
                              response.json['message'])
        return result

    if description:
        # Update the description of a Gist if requested
        gist.description = description

    if filenames:
        for filename in filenames:
            # File to update
            file_obj = gist.getFile(filename)
            if not file_obj:
                if remove:
                    return build_result(False, literals.UPDATE_RM_NF)
                if new:
                    # Upload a new file to gist
                    gistFile = model.GistFile()
                    gistFile.filename = filename
                    with open(os.path.join(filepath, filename), 'r') as f:
                        file_content = f.read()
                        gistFile.content = file_content
                    gist.addFile(gistFile)
                else:
                    # File not found and option --new it does not exist
                    return build_result(False, literals.UPDATE_NF)
            else:
                if new:
                    # File not found and option --new it does not exist
                    return build_result(False, literals.UPDATE_NEW_DUP)
                if remove:
                    # Remove a file
                    gist.setFile(filename, "null")
                else:
                    # Update the contents of the file
                    with open(os.path.join(filepath, filename), 'r') as f:
                        file_content = f.read()
                        file_obj.content = file_content
                    gist.setFile(filename, file_obj)

    # prepare the request
    response = facade.update_gist(gist)
    if response.ok:
        return build_result(True, gist)
    else:
        return build_result(False, literals.UPDATE_NOK,
                            response.json['message'])

    return result


def authorize(facade):
    """ Configure the user and password of the GitHub user.

    :param facade: The Github interface
    """

    # check if there is already an authorization for the app
    response = facade.list_authorizations()
    if response.ok:
        for auth in response.json:
            authorization = model.Authorization(auth)
            if authorization.note == literals.APP_NAME:
                # write the token to the configuration file
                configurer = GistsConfigurer()
                configurer.setConfigUser(facade.username)
                configurer.setConfigToken(authorization.token)
                return build_result(True, authorization)
    else:
        return build_result(False, literals.AUTHORIZE_NOK,
                            response.json['message'])

    # build the authorization request
    auth = model.Authorization()
    auth.note = literals.APP_NAME
    auth.note_url = literals.APP_URL
    auth.scopes = ["gist"]

    response = facade.authorize(auth)

    if response.ok:
        auth = model.Authorization(response.json)
        result = build_result(True, auth)

        # write the token to the configuration file
        configurer = GistsConfigurer()
        configurer.setConfigUser(facade.username)
        configurer.setConfigToken(auth.token)
    else:
        result = build_result(False, literals.AUTHORIZE_NOK,
                              response.json['message'])

    return result


def fork(gist_id, facade):
    """ Forks a gist.

    :param gistid: identifier of the Gist to fork
    :param facade: instance of the object that actually performs the request
    """
    response = facade.fork_gist(gist_id)

    if response.ok:
        result = build_result(True, model.Gist(response.json))
    else:
        if response.json:
            result = build_result(False, literals.FORK_ERROR, gist_id,
                                  response.json['message'])
        else:
            result = build_result(False, literals.UNHANDLED_EXCEPTION)
    return result


def star(gist_id, facade):
    """ Stars a gist.

    :param gistid: identifier of the Gist to star
    :param facade: instance of the object that actually performs the request
    """
    response = facade.star_gist(gist_id)

    if response.ok:
        result = build_result(True, literals.STAR_OK, gist_id)

    else:
        res_message = response.json['message']
        result = build_result(False, literals.STAR_NOK, res_message)

    return result


def unstar(gist_id, facade):
    """ Unstars a gist.

    :param gistid: identifier of the Gist to unstar
    :param facade: instance of the object that actually performs the request
    """
    response = facade.unstar_gist(gist_id)

    if response.ok:
        result = build_result(True, literals.UNSTAR_OK, gist_id)

    else:
        res_message = response.json['message']
        result = build_result(False, literals.UNSTAR_NOK, res_message)

    return result



# Copyright (c) 2012 <Jaume Devesa (jaumedevesa@gmail.com)>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""

gists.utils
~~~~~~~~~~~

This module provides utils class and functionallity such as
base64 encoding, 'requests' module encapsulation, result data model, etc...

"""

import json
import requests
import urllib2
import os
import literals
import ConfigParser
from clint.textui import colored


class Result(object):
    """ The :class: `Result <Result>`.

    This class is return type of all the methods in 'gists.actions' module.
    """
    def __init__(self):
        self.success = False
        self.data = None


class GithubFacade(object):
    """ :class: `GithubFacade <GithubFacade> executes all the calls to Github.

    Although it is a trivial class, it is useful to mock its behavior
    in tests and to encapsulate the 'requests' dependency
    """
    # Endpoints to gists' github API
    ENDPOINT_LIST = "https://api.github.com/users/%s/gists"
    ENDPOINT_STARRED = "https://api.github.com/gists/starred"
    ENDPOINT_GIST = "https://api.github.com/gists/%s"
    ENDPOINT_CREATE = "https://api.github.com/gists"
    ENDPOINT_AUTH = "https://api.github.com/authorizations"
    ENDPOINT_FORK = "https://api.github.com/gists/%s/fork"
    ENDPOINT_STAR = "https://api.github.com/gists/%s/star"

    # Default content type
    APPLICATION_JSON = "application/json"

    def __init__(self, username=None, credential=None):
        """ Initializes the Github facade.

        :param username: The username used to connect to the API. Can
                be None if using the token based authentication.
        :param credential: The password (if username provided) or the
                token to be used to authenticate.
        """
        self.username = username
        self.credential = credential
        self.basic_auth = (username is not None)

    def request_list_of_gists(self, username):
        """ Call to get a list of gists for user.

        """
        # Set the URL
        url = self.ENDPOINT_LIST % (username)

        if self.basic_auth and self.credential:
            return requests.get(url, auth=(self.username, self.credential))
        else:
            params = {}
            if self.credential:
                # Set the authentication header only if the password is set
                params = {'access_token': self.credential}
            return requests.get(url, params=params)

    def request_list_starred_gists(self, username):
        """ Call to get a list of gists for user.

        """
        # Set the URL
        url = self.ENDPOINT_STARRED

        if self.basic_auth and self.credential:
            return requests.get(url, auth=(self.username, self.credential))
        else:
            params = {}
            if self.credential:
                # Set the authentication header only if the password is set
                params = {'access_token': self.credential}
            return requests.get(url, params=params)

    def request_gist(self, id_gist):
        """ Request a single Gist info.

        :param id_gist: identifier of the gist.
        """

        # Set the URL and send the request
        url = self.ENDPOINT_GIST % (id_gist)
        return requests.get(url)

    def create_gist(self, payload):
        """ Create a new gist.

        :param payload: the data of the message body. It contains description,
            whenever is public or private and, of course, file contents.
        """

        url = self.ENDPOINT_CREATE
        headers = {'Content-type': self.APPLICATION_JSON}
        data_json = json.dumps(payload, indent=2)

        if self.basic_auth:
            return requests.post(url, data=data_json, headers=headers,
                                 auth=(self.username, self.credential))
        else:
            params = {'access_token': self.credential}
            return requests.post(url, data=data_json, headers=headers,
                                 params=params)

    def update_gist(self, payload):
        """ Update an existent Gist via PATCH method.

        :param payload: the data of the message body. It contains description,
            whenever is public or private and, of course, file contents.
        """

        url = self.ENDPOINT_GIST % (payload.identifier)
        headers = {'Content-type': self.APPLICATION_JSON}
        data_json = json.dumps(payload, indent=2)

        if self.basic_auth:
            return requests.patch(url, data=data_json, headers=headers,
                                  auth=(self.username, self.credential))
        else:
            params = {'access_token': self.credential}
            return requests.patch(url, data=data_json, headers=headers,
                                  params=params)

    def delete_gist(self, id_gist):
        """ Delete an existent Gist.

        :param id_gist: identifier of the Gist to delete
        """

        url = self.ENDPOINT_GIST % (id_gist)
        if self.basic_auth:
            return requests.delete(url,
                                   auth=(self.username, self.credential))
        else:
            params = {'access_token': self.credential}
            return requests.delete(url, params=params)

    def list_authorizations(self):
        """ List the authorizations for the given user. """

        return requests.get(self.ENDPOINT_AUTH,
                            auth=(self.username, self.credential))

    def fork_gist(self, gist_id):
        """ Requests to GitHub Gist API to fork a gist. """

        url = self.ENDPOINT_FORK % (gist_id)
        if self.basic_auth:
            return requests.post(url, auth=(self.username, self.credential))
        else:
            params = {'access_token': self.credential}
            return requests.post(url, params=params)

    def star_gist(self, gist_id):
        """ Requests to GitHub Gist API to star a gist. """

        url = self.ENDPOINT_STAR % (gist_id)
        if self.basic_auth:
            headers = {'Content-length': '0'}
            return requests.put(url, auth=(self.username, self.credential),
                                headers=headers)
        else:
            params = {'access_token': self.credential}
            headers = {'Content-length': '0'}
            return requests.put(url, params=params, headers=headers)

    def unstar_gist(self, gist_id):
        """ Requests to GitHub Gist API to unstar a gist. """

        url = self.ENDPOINT_STAR % (gist_id)
        if self.basic_auth:
            return requests.delete(url, auth=(self.username, self.credential))
        else:
            params = {'access_token': self.credential}
            return requests.delete(url, params=params)

    def authorize(self, payload):
        """ Authorize the current app.

        :param payload: The data of the message body. It contains the
            authorization request information.
        """
        url = self.ENDPOINT_AUTH
        headers = {'Content-type': self.APPLICATION_JSON}
        data_json = json.dumps(payload, indent=2)
        return requests.post(url, data=data_json, headers=headers,
                             auth=(self.username, self.credential))


class GistsConfigurer(object):
    """ The :class: `GistsConfigurer <GistsConfigurer>` is the module that
    sets and gets data from the configuration file '.gistsrc'
    """

    def __init__(self):
        """ Initializes the 'ConfigParser' instance.

        If it does not found the '~/.gistsrc' file from the current user,
        it creates it empty.
        """

        self.config = ConfigParser.ConfigParser()
        self.config_file_path = os.path.expanduser('~/.gistsrc')
        if os.path.exists(self.config_file_path):
            self.config.read(self.config_file_path)
        else:
            with open(self.config_file_path, 'w'):
                # just create the file
                pass

    def getConfigUser(self):
        """ Returns the user from the configuration file.

        If configuration instance con not load 'credentials' section,
        or user is not configured, it returns none.
        """

        if not self.config.has_section('credentials'):
            print literals.CONFIG_FILE_NOT_FOUND
            return None
        username = self.config.get('credentials', 'user')
        return username

    def setConfigUser(self, user):
        """ Sets the user into the configuration file. """

        if not self.config.has_section('credentials'):
            self.config.add_section('credentials')
        self.config.set('credentials', 'user', user)
        with open(self.config_file_path, 'w') as f:
            self.config.write(f)

    def getConfigToken(self):
        """ Returns the auth token from the configuration file.

        If configuration instance con not load 'credentials' section,
        or user is not configured, it returns none.
        """

        if not self.config.has_section('credentials'):
            print literals.CONFIG_FILE_NOT_FOUND
            return None
        username = self.config.get('credentials', 'token')
        return username

    def setConfigToken(self, token):
        """ Sets the auth token into the configuration file. """

        if not self.config.has_section('credentials'):
            self.config.add_section('credentials')
        self.config.set('credentials', 'token', token)
        with open(self.config_file_path, 'w') as f:
            self.config.write(f)


def download(url, destination_dir, file_name, file_size):
    """ Downloads a file.

    :param url: remote location of the file
    :param destination_dir: target directory where the file will be
        download
    :param file_name: name of the target file
    :param file_size: size of the file to download.
    """

    destination_path = os.path.join(destination_dir, file_name)
    print colored.green(literals.DOWNLOADING %
                        (url, destination_path, file_size))

    # Open the remote url as a file, read it and write it in
    # target directory
    u = urllib2.urlopen(url)
    with open(destination_path, 'wb') as f:
        raw_file = u.read()
        f.write(raw_file)


def build_result(success, data, *args):
    """ Builds the :class: `Result <Result>` as return type.

    :param success: whenever it has finished succesfully or not
    :param data: data to print (Gist data or error data)
    :param *args: parametrized extra data.
    """

    # Not too much magic here
    result = Result()
    result.success = success
    if not args:
        result.data = data
    else:
        result.data = data % (args)
    return result



# Copyright (c) 2012 <Jaume Devesa (jaumedevesa@gmail.com)>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""

gists.handlers
~~~~~~~~~~~~~~

Handlers module contain all the fuctions that hanlde the input arguments from
the command line, looking for incompatible arguments, and filling configuration
data for the functions in 'actions' module.

"""

import sys
import utils
import literals
import getpass


# Load the configuration instance once the module is imported
config = utils.GistsConfigurer()


def handle_list(args):
    """ Handle the arguments to call the 'list gists' functionality. """

    # Get the 'user' argument if exists, otherwise take it from configuration
    # file. If 'user' can not be loaded, raise an exception
    if args.user:
        username = args.user
    else:
        username = config.getConfigUser()
    if not username:
        print literals.USER_NOT_FOUND
        sys.exit()

    # If '--private' or '--starred' options, password becomes mandatory.
    # Load it.
    if args.private or args.starred:
        # Get the 'credentials' argument if exists, otherwise take it from
        # configuration file. If 'secret' can not be loaded, raise an exception
        credential = get_credentials(args)
    else:
        credential = None

    return (username, utils.GithubFacade(args.user, credential),
            args.starred)


def handle_update(args):
    """ Handle the arguments to call the 'update gist' functionality. """
    if not args.input_dir:
        args.input_dir = "./"

    return (args.gist_id, args.description, args.filenames,
            args.input_dir, args.new, args.remove,
            utils.GithubFacade(args.user, get_credentials(args)))


def handle_post(args):
    """ Handle the arguments to call the 'create gist' functionality. """

    # Define public or private
    if args.private:
        public = False
    else:
        public = True

    if not args.input_dir:
        args.input_dir = "./"

    return (public, args.filenames, args.input_dir, args.description,
            utils.GithubFacade(args.user, get_credentials(args)))


def handle_show(args):
    """ Handle the arguments to call the 'show' gists functionality. """
    return args.gist_id, args.filename, utils.GithubFacade()


def handle_get(args):
    """ Handle the arguments to call the 'get' gists functionality. """
    return args.gist_id, args.filename, args.output_dir, utils.GithubFacade()


def handle_delete(args):
    """ Handle the arguments to call the 'delete' gists functionality. """
    return args.gist_id, utils.GithubFacade(args.user, get_credentials(args))


def handle_authorize(args):
    """ Handle the arguments to call the 'authorize' gists functionality. """
    password = get_credentials(args)
    return utils.GithubFacade(args.user, password),


def handle_fork(args):
    """ Handle the arguments to call the 'fork' gists functionality. """
    return args.gist_id, utils.GithubFacade(args.user, get_credentials(args))


def handle_star(args):
    """ Handle the arguments to call the 'star' and 'unstar' gists
    functionality. """
    return args.gist_id, utils.GithubFacade(args.user, get_credentials(args))


def get_credentials(args):
    """ Get the credentials to authenticate through Github.

    If the argument 'user' has been supplied, then it prompts the password
    request.
    Otherwise it will look for the authentication token in the configuration
    file
    """
    if (args.user):
        credentials = getpass.getpass("Github password for user '%s': "
                                      % (args.user))
    else:
        credentials = config.getConfigToken()
    if not credentials:
        print literals.CREDENTIAL_NOT_FOUND
        sys.exit()

    return credentials



# Copyright (c) 2012 <Jaume Devesa (jaumedevesa@gmail.com)>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.


"""

gists.gists
~~~~~~~~~~~

This single-function module defines the input parameters and the subparsers,
and coordinates the 'handlers'->'actions'->'formatters' execution workflow

"""

import argparse
from actions import (list_gists, show, get, post, delete, update, authorize,
                     fork, star, unstar)
from handlers import (handle_list, handle_show, handle_update,
                      handle_authorize, handle_get, handle_post, handle_delete,
                      handle_fork, handle_star)
from formatters import (format_list, format_post, format_update,
                        format_get, format_show, format_delete,
                        format_authorize, format_star)
from version import VERSION


USER_MSG = ("github username. Use this user instead of the defined one in "
            "the configuration file. If action demands authentication, a "
            "password request will be prompt")
GIST_ID_MSG = ("identifier of the Gist. Execute 'gists list' to know Gists "
               "identifiers")


def run(*args, **kwargs):

    # Initialize argument's parser
    description = 'Manage Github gists from CLI'
    parser = argparse.ArgumentParser(description=description,
                                     epilog="Happy Gisting!")

    # Define subparsers to handle each action
    subparsers = parser.add_subparsers(help="Available commands.")

    # Add the subparsers
    __add_list_parser(subparsers)
    __add_show_parser(subparsers)
    __add_get_parser(subparsers)
    __add_create_parser(subparsers)
    __add_update_parser(subparsers)
    __add_delete_parser(subparsers)
    __add_authorize_parser(subparsers)
    __add_version_parser(subparsers)
    __add_fork_parser(subparsers)
    __add_star_parser(subparsers)
    __add_unstar_parser(subparsers)

    # Parse the arguments
    args = parser.parse_args()

    # Calling the handle_args function defined, parsing the args and return
    # and object with the needed values to execute the function
    parameters = args.handle_args(args)

    # Passing the 'parameters' object as array of parameters
    result = args.func(*parameters)

    # Parsing the 'result' object to be output formatted.
    # (that must be a single object)
    result_formatted = args.formatter(result)

    # Print the formatted output
    print result_formatted


def __add_list_parser(subparsers):
    """ Define the subparser to handle the 'list' functionality.

    :param subparsers: the subparser entity
    """

    # Add the subparser to handle the list of gists
    parser_list = subparsers.add_parser("list", help="list a user's Gists")
    parser_list.add_argument("-u", "--user", help=USER_MSG)
    group1 = parser_list.add_mutually_exclusive_group()
    group1.add_argument("-p", "--private", help="""return the private gists
                        besides the public ones. Needs authentication""",
                        action="store_true")
    group1.add_argument("-s", "--starred", help="""return ONLY the starred
                        gists. Needs authentication""", action="store_true")
    parser_list.set_defaults(handle_args=handle_list,
                             func=list_gists, formatter=format_list)


def __add_show_parser(subparsers):
    """ Define the subparser to handle with the 'show' functionallity.

    :param subparsers: the subparser entity
    """
    # Add the subparser to handle the 'show' action
    parser_show = subparsers.add_parser("show", help="""show a Gist. Shows
                                        Gist metadata by default.
                                        With '-f' (--filename) option, shows
                                        the content of one of the Gist files
                                        """)
    parser_show.add_argument("gist_id", help=GIST_ID_MSG)
    parser_show.add_argument("-f", "--filename", help="gist file to show")
    parser_show.set_defaults(handle_args=handle_show, func=show,
                             formatter=format_show)


def __add_get_parser(subparsers):
    """ Define the subparser to handle the 'get' functionality.

    :param subparsers: the subparser entity
    """

    # Add the subparser to handle the 'get' action
    parser_get = subparsers.add_parser("get", help="""download a single gist
                                       file. If the gist has just a single
                                       file, argument '-f' (--filename) is not
                                       needed""")
    parser_get.add_argument("gist_id", help=GIST_ID_MSG)
    parser_get.add_argument("-f", "--filename", help="file to download")
    parser_get.add_argument("-o", "--output_dir", help="destination directory",
                            default=".")
    parser_get.set_defaults(handle_args=handle_get, func=get,
                            formatter=format_get)


def __add_create_parser(subparsers):
    """ Define the subparser to handle the 'create' functionality.

    :param subparsers: the subparser entity
    """

    # Add the subparser to handle the 'create' action
    parser_post = subparsers.add_parser("create", help="""create a new gist.
                                        Needs authentication""")
    parser_post.add_argument("-u", "--user", help=USER_MSG)
    parser_post.add_argument("-f", "--filenames", nargs='+', help="""specify
                             files to upload with Gist creation""",
                             required=True)
    parser_post.add_argument("-p", "--private", help="""private Gist? ('false'
                             by default)""", action="store_true")
    parser_post.add_argument("-i", "--input_dir", help="""input directory where
                             the source files are""")
    parser_post.add_argument("-d", "--description", help="""description for
                             the Gist to create""")
    parser_post.set_defaults(handle_args=handle_post, func=post,
                             formatter=format_post)


def __add_update_parser(subparsers):
    """ Define the subparser to handle the 'update' functionality.

    :param subparsers: the subparser entity
    """

    # Add the subparser to handle the 'update' action
    parser_update = subparsers.add_parser("update", help="""update a gist.
                                          Needs authentication""")
    parser_update.add_argument("gist_id", help=GIST_ID_MSG)
    parser_update.add_argument("-u", "--user", help=USER_MSG)

    group1 = parser_update.add_argument_group("file options",
                                              "update Gist files")
    group1.add_argument("-f", "--filenames", nargs='+',
                        help="Gist files to update")
    group11 = group1.add_mutually_exclusive_group()
    group11.add_argument("-n", "--new", action="store_true", help="""files
                         supplied are new for the Gist. '-f' (--filenames)
                         argument needed""",
                         default=False)
    group11.add_argument("-r", "--remove", action="store_true",
                         help="""files supplied will be removed from the Gist.
                         '-f' (--filenames) argument needed""", default=False)
    group1.add_argument("-i", "--input_dir", help="""directory where the files
                        are. Current directory by default""")
    group2 = parser_update.add_argument_group('metadata options',
                                              "update Gist metadata")
    group2.add_argument("-d", "--description", help="update Gist description")
    parser_update.set_defaults(handle_args=handle_update, func=update,
                               formatter=format_update)


def __add_delete_parser(subparsers):
    """ Define the subparser to handle the 'delete' functionality.

    :param subparsers: the subparser entity
    """

    # Add the subparser to handle the 'delete' action
    parser_delete = subparsers.add_parser("delete", help="""delete a Gist.
                                          Needs authentication""")
    parser_delete.add_argument("gist_id", help=GIST_ID_MSG)
    parser_delete.add_argument("-u", "--user", help=USER_MSG)
    parser_delete.set_defaults(handle_args=handle_delete, func=delete,
                               formatter=format_delete)


def __add_authorize_parser(subparsers):
    """ Define the subparser to handle the 'authorize' functionallity.

    :param subparsers: the subparser entity
    """

    # Add the subparser to handle the 'authorize' action.
    parser_authorize = subparsers.add_parser("authorize", help="""authorize
                                             this project in github""")
    parser_authorize.add_argument("-u", "--user", help="""your github user
                                  . Needed to generate the auth token. """,
                                  required=True)
    parser_authorize.set_defaults(handle_args=handle_authorize, func=authorize,
                                  formatter=format_authorize)


def __add_version_parser(subparsers):
    """ Define the subparser to handle 'version' functionallity.

    :param subparsers: the subparser entity
    """

    parser_version = subparsers.add_parser("version", help="""print the version
                                           of the release""")
    parser_version.set_defaults(handle_args=lambda x: (None,),
                                func=lambda x: None,
                                formatter=lambda x: VERSION)


def __add_fork_parser(subparsers):
    """ Define the subparser to handle 'fork' functionallity.

    :param subparsers: the subparser entity
    """

    parser_fork = subparsers.add_parser("fork", help="""fork another users'
                                        Gists""")
    parser_fork.add_argument("gist_id", help=GIST_ID_MSG)
    parser_fork.add_argument("-u", "--user", help=USER_MSG)
    parser_fork.set_defaults(handle_args=handle_fork, func=fork,
                             formatter=format_post)


def __add_star_parser(subparsers):
    """ Define the subparser to handle 'star' functionallity.

    :param subparsers: the subparser entity
    """

    parser_star = subparsers.add_parser("star", help="star a Gist")
    parser_star.add_argument("gist_id", help=GIST_ID_MSG)
    parser_star.add_argument("-u", "--user", help=USER_MSG)
    parser_star.set_defaults(handle_args=handle_star, func=star,
                             formatter=format_star)


def __add_unstar_parser(subparsers):
    """ Define the subparser to handle 'unstar' functionallity.

    :param subparsers: the subparser entity
    """

    parser_unstar = subparsers.add_parser("unstar", help="unstar a Gist")
    parser_unstar.add_argument("gist_id", help=GIST_ID_MSG)
    parser_unstar.add_argument("-u", "--user", help=USER_MSG)
    parser_unstar.set_defaults(handle_args=handle_star, func=unstar,
                               formatter=format_star)



# Copyright (c) 2012 <Jaume Devesa (jaumedevesa@gmail.com)>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""

gists.literals
~~~~~~~~~~~

All the return literals here.

"""

APP_NAME = "Gists CLI"

APP_URL = "https://github.com/jdevesa/gists"

LISTS_ERROR = "Can not return the list of gists. Github reason: '%s'"

DOWNLOAD_OK = "File '%s' downloaded successfully!"

DOWNLOAD_MORE_FILES = ("Gist has more than one file. "
                       """Specify file by '-f' option. Available values are
                       [%s] """)

FILE_NOT_FOUND = ("File not found in gist. Available values are [%s])")

DOWNLOAD_ERROR = "Can not download gist file. Github reason: '%s'"

SHOW_ERROR = "Can not show gist file. Github reason: '%s'"

UNHANDLED_EXCEPTION = "Unhandled exception"

DELETE_CONFIRMATION = "Are you sure you want to delete gist %s [yN]"

DELETE_OK = "Gist '%s' deleted succesfully"

DELETE_NOK = "Can not delete the gist. Github reason: '%s'"

DELETE_ABORTED = "Delete aborted."

UPDATE_NOK = "Can not update the gist. Github reason: '%s'"

UPDATE_RM_NF = "Can not remove a file that actually does not exist in gist. "

UPDATE_NF = ("Filename not found in gist. Use the '-n' (--new) argument "
             "to attach a new file in the gist.")

UPDATE_NEW_DUP = """File already exists in Gist. Remove the 'n' (--new)
                 argument if you want to update the existent file. Change the
                 file name if you actually want to update a new file in the
                 gist. """

CONFIG_FILE_NOT_FOUND = "Configuration file not found or not valid"

DOWNLOADING = "Downloading: %s\nTo: %s\nBytes: %s"

USER_NOT_FOUND = """Can not load GitHub user name neither from '--user (-u)'
                 parameter nor from the configuration file. """

CREDENTIAL_NOT_FOUND = """Credential should be informed as a token in
                       configuration file or by prompt when using '-u' (--user)
                       argument"""

AUTHORIZE_NOK = "Could not get an authorization token. Github reason: '%s'"

FORK_ERROR = "Can not fork the gist '%s'. Github reason: '%s'"

STAR_OK = "Gist '%s' starred succesfully"

STAR_NOK = "Can not star the gist. Github reason: '%s'"

UNSTAR_OK = "Gist '%s' unstarred succesfully"

UNSTAR_NOK = "Can not unstar the gist. Github reason: '%s'"



import os
import sys
from setuptools import setup

README = open('README').read()


def get_version():
    """ Copied from @nvie:

    https://github.com/nvie/rq/blob/master/setup.py
    I'm toooooo lazy ...
    """
    basedir = os.path.dirname(__file__)
    with open(os.path.join(basedir, 'gists/version.py')) as f:
        VERSION = None
        exec(f.read())
        return VERSION
    raise RuntimeError('No version info found.')

requirements = ['requests == 0.14.0', 'clint == 0.3.1']
if sys.version_info < (2, 7):
    requirements.append('argparse')
elif sys.version_info < (2, 6):
    raise 'Must use python 2.6 or greater'

setup(name='gists',
      packages=['gists'],
      version=get_version(),
      author='Jaume Devesa',
      author_email='jaumedevesa@gmail.com',
      url='http://jdevesa.github.com/gists',
      description='CLI interface to manage Github gists',
      install_requires=requirements,
      scripts=['gists/gists'],
      include_package_data=True,
      long_description=README,
      license='MIT',
      test_suite='tests',
      classifiers=[
           "Programming Language :: Python",
           "Programming Language :: Python :: 2.6",
           "Programming Language :: Python :: 2.7",
           "Development Status :: 5 - Production/Stable ",
           "Environment :: Console",
           "Intended Audience :: Developers",
           "License :: OSI Approved :: MIT License",
           "Natural Language :: English",
           "Operating System :: POSIX :: Linux"
      ])



"""

gists
~~~~~

:copyright: (c) 2012 by Jaume Devesa
:license: MIT, see LICENSE for more details

"""

__title__ = 'gists'
__version__ = '0.1'
__author__ = 'Jaume Devesa'
__license__ = 'MIT'
__copyright__ = 'Copyright 2012 Jaume Devesa'



# Copyright (c) 2012 <Jaume Devesa (jaumedevesa@gmail.com)>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""

gists.model
~~~~~~~~~~~

Model defines the two classes that extend of builtin python 'dicts'.
They represent a Gist object and a Gist File. Used to make the code of other
modules clearer.

"""


class Gist(dict):
    """ :class: `Gist <Gist>` Simple Gist object. """

    def __init__(self, parsed_gist={}):
        """ Initialize gist object variables. """

        super(Gist, self).__init__(parsed_gist)

    @property
    def url(self):
        return self['url']

    @property
    def html_url(self):
        return self['html_url']

    @property
    def description(self):
        return self['description']

    @description.setter
    def description(self, description):
        self['description'] = description

    @property
    def identifier(self):
        return self['id']

    @property
    def user(self):
        return self['user']['login']

    @property
    def public(self):
        return self['public']

    @public.setter
    def public(self, public):
        self['public'] = public

    @property
    def files(self):
        """ Parse the 'self['files']' into GistFile objects. """
        if not 'files' in self:
            self['files'] = {}
            return self['files']
        return [GistFile(self['files'][gistfile])
                for gistfile in self['files']
                if self['files'][gistfile] != 'null']

    def getFile(self, requested_filename):
        candidates = [gistfile for gistfile in self.files
                      if gistfile.filename == requested_filename]
        if len(candidates) == 0:
            return None
        return candidates[0]

    def setFile(self, filename, gist_file):
        self['files'][filename] = gist_file

    def addFile(self, gistfile):
        if not 'files' in self:
            self['files'] = {}
        self['files'][gistfile.filename] = gistfile


class GistFile(dict):
    """ :class: `GistFile <GistFile>` File that belongs to a Gist. """

    def __init__(self, parsed_file={}):
        super(GistFile, self).__init__(parsed_file)

    @property
    def raw_url(self):
        return self['raw_url']

    @property
    def language(self):
        return self['language']

    @property
    def filename(self):
        return self['filename']

    @filename.setter
    def filename(self, filename):
        self['filename'] = filename

    @property
    def content(self):
        return self['content']

    @content.setter
    def content(self, content):
        self['content'] = content

    @property
    def mimetype(self):
        return self['type']

    @property
    def size(self):
        return self['size']


class Authorization(dict):
    """ :class: `Authorization <Authorization>` authorization object. """

    def __init__(self, parsed_auth={}):
        super(Authorization, self).__init__(parsed_auth)

    @property
    def id(self):
        return self['id']

    @property
    def url(self):
        return self['url']

    @property
    def app(self):
        return self['app']['name']

    @property
    def app_url(self):
        return self['app']['url']

    @property
    def created(self):
        return self['created_at']

    @property
    def updated(self):
        return self['updated_at']

    @property
    def token(self):
        return self['token']

    @property
    def note(self):
        return self['note']

    @note.setter
    def note(self, note):
        self['note'] = note

    @property
    def note_url(self):
        return self['note_url']

    @note_url.setter
    def note_url(self, note_url):
        self['note_url'] = note_url

    @property
    def scopes(self):
        return self['scopes']

    @scopes.setter
    def scopes(self, scopes):
        self['scopes'] = scopes



# Copyright (c) 2012 <Jaume Devesa (jaumedevesa@gmail.com)>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import os
import model
from clint.textui import colored

"""

gists.formatters
~~~~~~~~~~~~~~~~

'formatters' is the last step of the execution workflow. All the methods
receive an instance of the :class: `Result <Result>` and use them to format
the output legible, clear and pretty (using the usefull 'clint' package)


"""


def format_show(result):
    """ Formats the output of the 'show' action.

    :param result: Result instance
    """

    if result.success:
        resultdata = result.data

        # If the data is an instance of the 'GistFile' data model,
        # parse the file, otherwise, parse the 'Gist' metadata
        if isinstance(resultdata, model.GistFile):
            return __format_file(resultdata)
        else:
            return __format_gist(resultdata)
    else:
        # Format the error string message
        return __format_error(result.data)


def format_post(result):
    """ Formats the output of the 'create' action.

    :param result: Result instance
    """

    if result.success:
        # Format the 'Gist' metadata object
        return __format_gist(result.data)
    else:
        # Format the error string message
        return __format_error(result.data)


def format_get(result):
    """ Formats the output of the 'get/download' action.

    :param result: Result instance
    """

    if result.success:
        # The result is just a string informing the success
        return result.data
    else:
        # Format the error string message
        return __format_error(result.data)


def format_delete(result):
    """ Formats the output of the 'delete' action.

    :param result: Result instance
    """

    if result.success:
        # The result is just a string informing the success
        return result.data
    else:
        # Format the error string message
        return __format_error(result.data)


def format_update(result):
    """ Formats the output of the 'delete' action.

    :param result: Result instance
    """

    if result.success:
        # Format the 'Gist' metadata object
        return __format_gist(result.data)
    else:
        # Format the error string message
        return __format_error(result.data)


def format_list(result):
    """ Formats the output of the 'list' action.

    :param result: Result instance
    """
    if result.success:

        # Get the list of Gists from the data
        list_of_gists = result.data

        # Calculate the number of columns of the current terminal window
        rows, columns = os.popen('stty size', 'r').read().split()

        # Set the header
        gists_string = colored.cyan('-' * int(columns)) + "\n"
        gists_string += colored.cyan("List of gists\n")
        gists_string += colored.cyan('-' * int(columns)) + "\n"

        # Set the contents for each Gist listed
        for gist in list_of_gists:
            gists_string += colored.green(gist.identifier + ": ")
            description = "(no desc)"
            if gist.description and gist.description != "":
                description = gist.description
            gists_string += description
            gist_names = [gistfile.filename for
                          gistfile in gist.files]
            stringfiles = " [" + ", ".join(gist_names) + "]"
            gists_string += colored.red(stringfiles)
            if not gist.public:
                gists_string += " (Private Gist) "
            gists_string += '\n'

        # Set the footer
        gists_string += colored.cyan('-' * int(columns)) + "\n"

        # Return the formatted String
        return gists_string
    else:
        # Format the error string message
        return __format_error(result.data)


def format_authorize(result):
    """ This is enough for this method. """
    if result.success:
        return colored.green("Authentication token written in '~/.gistsrc'")
    else:
        # Format the error string message
        return __format_error(result.data)


def format_star(result):
    """ Formats the output of the 'star' and 'unstar' action.

    :param result: Result instance
    """

    if result.success:
        # The result is just a string informing the success
        return result.data
    else:
        # Format the error string message
        return __format_error(result.data)


def __format_gist(gist):
    """ Formats the output for a Gist metadata object.

    :param gist: :class: `Gist <Gist>` instance.
    """
    url_title = unicode(colored.green('Url:\t\t'))
    html_title = unicode(colored.green('Html Url:\t'))
    public_title = unicode(colored.green('Private:\t'))
    file_title = unicode(colored.green('Files:\t\t'))

    # Calculate the number of columns of the current terminal window
    rows, columns = os.popen('stty size', 'r').read().split()

    # Prepare the Header
    gists_string = colored.cyan('-' * int(columns)) + "\n"
    gists_string += colored.cyan("Gist [" + gist.identifier + "]") + '\n'
    gists_string += colored.cyan('-' * int(columns)) + "\n"

    # Format Gist data
    gists_string += colored.green('Description:\t')
    if gist.description:
        gists_string += gist.description + '\n'
    gists_string += url_title + gist.url + '\n'
    gists_string += html_title + gist.html_url + '\n'
    gists_string += public_title + str(not gist.public) + '\n'
    gist_names = [gistfile.filename for gistfile in gist.files]
    stringfiles = "[" + ", ".join(gist_names) + "]"
    gists_string += file_title + colored.red(stringfiles) + '\n'

    # Prepare the Footer
    gists_string += colored.cyan('-' * int(columns)) + "\n"

    return gists_string


def __format_error(data):
    """ Print the string output error. """
    return colored.red("Error: ") + data


def __format_file(file_gist):
    """ Formats the output for a GistFile object.

    :param gist: :class: `GistFile <GistFile>` instance.
    """

    # Calculate the number of columns of the current terminal window
    rows, columns = os.popen('stty size', 'r').read().split()

    # Prepare the Header
    gist_string = colored.cyan('-' * int(columns)) + "\n"
    gist_string += colored.cyan("File [" + file_gist.filename + "]\n")
    gist_string += colored.cyan('-' * int(columns)) + "\n"

    # Format Gist data
    gist_string += (colored.green("Language:") + " " +
                    colored.red(file_gist.language) + "\n")
    gist_string += (colored.green("Size:") + " " +
                    colored.red(file_gist.size) + "\n")
    gist_string += (colored.green("Raw Url:") + " " +
                    colored.red(file_gist.raw_url + "\n"))
    gist_string += (colored.green("Content:\n\n")
                    + file_gist.content + "\n\n")

    # Prepare the Footer
    gist_string += colored.cyan('-' * int(columns)) + "\n"

    return gist_string
